@@ -1,8 +1,5 @@
-(******************************************************************************)
-(* Lightweight thread library for OCaml
+(* OCaml promise library
  * http://www.ocsigen.org/lwt
- * Module Lwt_condition
- ******************************************************************************
  * Copyright (c) 2009, Metaweb Technologies, Inc.
  * All rights reserved.
  *
@@ -27,25 +24,34 @@
  * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
  * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- ******************************************************************************)
+ *)
+
+(* [Lwt_sequence] is deprecated – we don't want users outside Lwt using it.
+   However, it is still used internally by Lwt. So, briefly disable warning 3
+   ("deprecated"), and create a local, non-deprecated alias for
+   [Lwt_sequence] that can be referred to by the rest of the code in this
+   module without triggering any more warnings. *)
+[@@@ocaml.warning "-3"]
+module Lwt_sequence = Lwt_sequence
+[@@@ocaml.warning "+3"]
 
 type 'a t = 'a Lwt.u Lwt_sequence.t
 
 let create = Lwt_sequence.create
 
 let wait ?mutex cvar =
-  let waiter = Lwt.add_task_r cvar in
+  let waiter = (Lwt.add_task_r [@ocaml.warning "-3"]) cvar in
   let () =
     match mutex with
-      | Some m -> Lwt_mutex.unlock m
-      | None -> ()
+    | Some m -> Lwt_mutex.unlock m
+    | None -> ()
   in
   Lwt.finalize
     (fun () -> waiter)
     (fun () ->
        match mutex with
-         | Some m -> Lwt_mutex.lock m
-         | None -> Lwt.return_unit)
+       | Some m -> Lwt_mutex.lock m
+       | None -> Lwt.return_unit)
 
 let signal cvar arg =
   try
