@@ -1,24 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Interface Lwt_switch
- * Copyright (C) 2010 Jérémiem Dimino
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 (** Lwt switches *)
 
@@ -48,8 +31,8 @@
       ...
     ]}
 
-    However, one may want to handle possible failures of [f ()], [g
-    ()] and [h ()], and disable all allocated resources if one of
+    However, one may want to handle possible failures of [f ()], [g ()]
+    and [h ()], and disable all allocated resources if one of
     these three threads fails. This may be hard since you have to
     remember which one failed and which one returned correctly.
 
