@@ -3,14 +3,12 @@
 
 
 
-#include <lwt_config.h>
+#include "lwt_config.h"
 
 #if defined(LWT_ON_WINDOWS)
 
 #include <lwt_unix.h>
 
-#define CAML_NAME_SPACE
-#include <caml/version.h>
 #if OCAML_VERSION < 41300
 #define CAML_INTERNALS
 #endif
@@ -22,8 +20,8 @@
 
 static HANDLE get_handle(value opt) {
   value fd;
-  if (Is_block(opt)) {
-    fd = Field(opt, 0);
+  if (Is_some(opt)) {
+    fd = Some_val(opt);
     if (Descr_kind_val(fd) == KIND_SOCKET) {
       win32_maperr(ERROR_INVALID_HANDLE);
       uerror("CreateProcess", Nothing);
@@ -34,6 +32,31 @@ static HANDLE get_handle(value opt) {
     return INVALID_HANDLE_VALUE;
 }
 
+/* Ensures the handle [h] is inheritable. Returns the handle for the
+   child process in [hStd] and in [to_close] if it needs to be closed
+   after CreateProcess. */
+static int ensure_inheritable(HANDLE h /* in */,
+                              HANDLE * hStd /* out */,
+                              HANDLE * to_close /* out */)
+{
+  DWORD flags;
+  HANDLE hp;
+
+  if (h == INVALID_HANDLE_VALUE || h == NULL)
+    return 1;
+  if (! GetHandleInformation(h, &flags))
+    return 0;
+  hp = GetCurrentProcess();
+  if (! (flags & HANDLE_FLAG_INHERIT)) {
+    if (! DuplicateHandle(hp, h, hp, hStd, 0, TRUE, DUPLICATE_SAME_ACCESS))
+      return 0;
+    *to_close = *hStd;
+  } else {
+    *hStd = h;
+  }
+  return 1;
+}
+
 CAMLprim value lwt_process_create_process(value prog, value cmdline, value env,
                                           value cwd, value fds) {
   CAMLparam5(prog, cmdline, env, cwd, fds);
@@ -41,8 +64,29 @@ CAMLprim value lwt_process_create_process(value prog, value cmdline, value env,
 
   STARTUPINFO si;
   PROCESS_INFORMATION pi;
-  DWORD flags = CREATE_UNICODE_ENVIRONMENT;
-  BOOL ret;
+  DWORD flags = 0, err;
+  HANDLE hp, fd0, fd1, fd2;
+  HANDLE to_close0 = INVALID_HANDLE_VALUE, to_close1 = INVALID_HANDLE_VALUE,
+    to_close2 = INVALID_HANDLE_VALUE;
+
+  fd0 = get_handle(Field(fds, 0));
+  fd1 = get_handle(Field(fds, 1));
+  fd2 = get_handle(Field(fds, 2));
+
+  err = ERROR_SUCCESS;
+  ZeroMemory(&si, sizeof(si));
+  ZeroMemory(&pi, sizeof(pi));
+  si.cb = sizeof(si);
+  si.dwFlags = STARTF_USESTDHANDLES;
+
+  /* If needed, duplicate the handles fd1, fd2, fd3 to make sure they
+     are inheritable. */
+  if (! ensure_inheritable(fd0, &si.hStdInput, &to_close0) ||
+      ! ensure_inheritable(fd1, &si.hStdOutput, &to_close1) ||
+      ! ensure_inheritable(fd2, &si.hStdError, &to_close2)) {
+    err = GetLastError(); goto ret;
+  }
+
 
 #define string_option(opt) \
   (Is_block(opt) ? caml_stat_strdup_to_os(String_val(Field(opt, 0))) : NULL)
@@ -55,23 +99,25 @@ CAMLprim value lwt_process_create_process(value prog, value cmdline, value env,
 
 #undef string_option
 
-  ZeroMemory(&si, sizeof(si));
-  ZeroMemory(&pi, sizeof(pi));
-  si.cb = sizeof(si);
-  si.dwFlags = STARTF_USESTDHANDLES;
-  si.hStdInput = get_handle(Field(fds, 0));
-  si.hStdOutput = get_handle(Field(fds, 1));
-  si.hStdError = get_handle(Field(fds, 2));
+  flags |= CREATE_UNICODE_ENVIRONMENT;
+  if (! CreateProcess(progs, cmdlines, NULL, NULL, TRUE, flags,
+                      envs, cwds, &si, &pi)) {
+    err = GetLastError();
+  }
 
-  ret = CreateProcess(progs, cmdlines, NULL, NULL, TRUE, flags,
-                      envs, cwds, &si, &pi);
   caml_stat_free(progs);
   caml_stat_free(cmdlines);
   caml_stat_free(envs);
   caml_stat_free(cwds);
 
-  if (!ret) {
-    win32_maperr(GetLastError());
+ret:
+/* Close the handles if we duplicated them above. */
+  if (to_close0 != INVALID_HANDLE_VALUE) CloseHandle(to_close0);
+  if (to_close1 != INVALID_HANDLE_VALUE) CloseHandle(to_close1);
+  if (to_close2 != INVALID_HANDLE_VALUE) CloseHandle(to_close2);
+
+  if (err != ERROR_SUCCESS) {
+    win32_maperr(err);
     uerror("CreateProcess", Nothing);
   }
 
