@@ -1,3 +1,3 @@
 let _ =
   let%lwt () = Lwt.return 5 in
-  Lwt.return ();;
+  Lwt.return_unit;;
