@@ -1,2 +1,2 @@
 (* On one line for error message compatibility with 4.09. *)
-let%lwt () = Lwt.return ()
+let%lwt () = Lwt.return_unit
